/* Copyright 2012 Tobias Marschall
 * 
 * This file is part of CLEVER.
 * 
 * CLEVER is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CLEVER is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CLEVER.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef EDGEWRITER_H_
#define EDGEWRITER_H_

#include <iostream>
#include <deque>
#include <boost/unordered_set.hpp>

#include "PackedAlignmentPair.h"
#include "Types.h"

class EdgeWriter {
	std::ostream& os;
	typedef boost::unordered_set<alignment_id_t> node_set_t;
	std::deque<node_set_t*> nodes;
	size_t offset;
	bool finished;
	void printNode(alignment_id_t id);
public:
	EdgeWriter(std::ostream& os);
	virtual ~EdgeWriter();

	/** Sets an edge between the two given nodes. */
	virtual void addEdge(const PackedAlignmentPair& node1, const PackedAlignmentPair& node2);

	/** Calling this method indicates that the given node can be written to output
	 *  as no more edges will be added. */
	virtual void setNodeCompleted(const PackedAlignmentPair& node);

	/** Output all nodes and edges that are still unfinished. */
	virtual void finish();
};

#endif /* EDGEWRITER_H_ */
